@@ -15,21 +15,20 @@
 # You should have received a copy of the GNU General Public License
 # along with Ansible.  If not, see <https://www.gnu.org/licenses/>.
 
-# Make coding more python3-ish
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 import hashlib
 import os
 import string
 
+from collections.abc import Mapping
+
 from ansible.errors import AnsibleError, AnsibleParserError
 from ansible.inventory.group import to_safe_group_name as original_safe
 from ansible.parsing.utils.addresses import parse_address
 from ansible.plugins import AnsiblePlugin
 from ansible.plugins.cache import CachePluginAdjudicator as CacheObject
-from ansible.module_utils._text import to_bytes, to_native
-from ansible.module_utils.common._collections_compat import Mapping
+from ansible.module_utils.common.text.converters import to_bytes, to_native
 from ansible.module_utils.parsing.convert_bool import boolean
 from ansible.module_utils.six import string_types
 from ansible.template import Templar
@@ -219,7 +218,7 @@ class BaseInventoryPlugin(AnsiblePlugin):
         try:
             # avoid loader cache so meta: refresh_inventory can pick up config changes
             # if we read more than once, fs cache should be good enough
-            config = self.loader.load_from_file(path, cache=False)
+            config = self.loader.load_from_file(path, cache='none')
         except Exception as e:
             raise AnsibleParserError(to_native(e))
 
@@ -290,39 +289,13 @@ class BaseFileInventoryPlugin(BaseInventoryPlugin):
         super(BaseFileInventoryPlugin, self).__init__()
 
 
-class DeprecatedCache(object):
-    def __init__(self, real_cacheable):
-        self.real_cacheable = real_cacheable
-
-    def get(self, key):
-        display.deprecated('InventoryModule should utilize self._cache as a dict instead of self.cache. '
-                           'When expecting a KeyError, use self._cache[key] instead of using self.cache.get(key). '
-                           'self._cache is a dictionary and will return a default value instead of raising a KeyError '
-                           'when the key does not exist', version='2.12', collection_name='ansible.builtin')
-        return self.real_cacheable._cache[key]
-
-    def set(self, key, value):
-        display.deprecated('InventoryModule should utilize self._cache as a dict instead of self.cache. '
-                           'To set the self._cache dictionary, use self._cache[key] = value instead of self.cache.set(key, value). '
-                           'To force update the underlying cache plugin with the contents of self._cache before parse() is complete, '
-                           'call self.set_cache_plugin and it will use the self._cache dictionary to update the cache plugin',
-                           version='2.12', collection_name='ansible.builtin')
-        self.real_cacheable._cache[key] = value
-        self.real_cacheable.set_cache_plugin()
-
-    def __getattr__(self, name):
-        display.deprecated('InventoryModule should utilize self._cache instead of self.cache',
-                           version='2.12', collection_name='ansible.builtin')
-        return self.real_cacheable._cache.__getattribute__(name)
-
-
 class Cacheable(object):
 
     _cache = CacheObject()
 
     @property
     def cache(self):
-        return DeprecatedCache(self)
+        return self._cache
 
     def load_cache_plugin(self):
         plugin_name = self.get_option('cache_plugin')
@@ -358,7 +331,7 @@ class Cacheable(object):
 
 class Constructable(object):
 
-    def _compose(self, template, variables):
+    def _compose(self, template, variables, disable_lookups=True):
         ''' helper method for plugins to compose variables for Ansible based on jinja2 expression and inventory vars'''
         t = self.templar
 
@@ -372,7 +345,8 @@ class Constructable(object):
         else:
             t.available_variables = variables
 
-        return t.template('%s%s%s' % (t.environment.variable_start_string, template, t.environment.variable_end_string), disable_lookups=True)
+        return t.template('%s%s%s' % (t.environment.variable_start_string, template, t.environment.variable_end_string),
+                          disable_lookups=disable_lookups)
 
     def _set_composite_vars(self, compose, variables, host, strict=False):
         ''' loops over compose entries to create vars for hosts '''
@@ -423,8 +397,11 @@ class Constructable(object):
                         if strict:
                             raise AnsibleParserError("Could not generate group for host %s from %s entry: %s" % (host, keyed.get('key'), to_native(e)))
                         continue
-
-                    if key:
+                    default_value_name = keyed.get('default_value', None)
+                    trailing_separator = keyed.get('trailing_separator')
+                    if trailing_separator is not None and default_value_name is not None:
+                        raise AnsibleParserError("parameters are mutually exclusive for keyed groups: default_value|trailing_separator")
+                    if key or (key == '' and default_value_name is not None):
                         prefix = keyed.get('prefix', '')
                         sep = keyed.get('separator', '_')
                         raw_parent_name = keyed.get('parent_group', None)
@@ -438,14 +415,28 @@ class Constructable(object):
 
                         new_raw_group_names = []
                         if isinstance(key, string_types):
-                            new_raw_group_names.append(key)
+                            # if key is empty, 'default_value' will be used as group name
+                            if key == '' and default_value_name is not None:
+                                new_raw_group_names.append(default_value_name)
+                            else:
+                                new_raw_group_names.append(key)
                         elif isinstance(key, list):
                             for name in key:
-                                new_raw_group_names.append(name)
+                                # if list item is empty, 'default_value' will be used as group name
+                                if name == '' and default_value_name is not None:
+                                    new_raw_group_names.append(default_value_name)
+                                else:
+                                    new_raw_group_names.append(name)
                         elif isinstance(key, Mapping):
                             for (gname, gval) in key.items():
-                                name = '%s%s%s' % (gname, sep, gval)
-                                new_raw_group_names.append(name)
+                                bare_name = '%s%s%s' % (gname, sep, gval)
+                                if gval == '':
+                                    # key's value is empty
+                                    if default_value_name is not None:
+                                        bare_name = '%s%s%s' % (gname, sep, default_value_name)
+                                    elif trailing_separator is False:
+                                        bare_name = gname
+                                new_raw_group_names.append(bare_name)
                         else:
                             raise AnsibleParserError("Invalid group name format, expected a string or a list of them or dictionary, got: %s" % type(key))
 
